@@ -1,10 +1,24 @@
+from __future__ import annotations
+
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from .tokens import Token
+
+
 class Line:
     """
     A single output line not including the final newline.
 
     """
 
-    def __init__(self, tokens=None, level=0, offset=0, ignore=False):
+    def __init__(
+        self,
+        tokens: list[Token.BaseToken] | None = None,
+        level: int = 0,
+        offset: int = 0,
+        ignore: bool = False,
+    ) -> None:
         """
         Lines are currently never instantiated with arguments, but
         that doesn't mean they can't.
@@ -15,7 +29,7 @@ class Line:
         self.offset = offset
         self.ignore = ignore
 
-    def append(self, token):
+    def append(self, token: Token.BaseToken) -> None:
         """
         Append token to line.
 
@@ -23,7 +37,7 @@ class Line:
         self.tokens.append(token)
 
     @property
-    def text(self):
+    def text(self) -> str:
         """
         The text of this line including the original
         leading/trailing spaces.
@@ -32,7 +46,7 @@ class Line:
         return "".join([token.text for token in self.tokens])
 
     @property
-    def indents(self):
+    def indents(self) -> bool:
         """
         Whether this line has more opening than closing tokens.
 
@@ -41,7 +55,7 @@ class Line:
             [token for token in self.tokens if token.dedents]
         )
 
-    def indent(self, tabwidth):
+    def indent(self, tabwidth: int) -> str:
         """
         The final, indented text of this line.
 
@@ -52,7 +66,7 @@ class Line:
             return " " * (tabwidth * self.level + self.offset) + text
         return ""
 
-    def __len__(self):
+    def __len__(self) -> int:
         """
         The length of the line (so far), excluding the whitespace
         at the beginning. Be careful calling len() because it might
@@ -62,7 +76,7 @@ class Line:
         """
         return len(self.text.lstrip())
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         kwargs = ""
         for attr in ["level", "offset", "ignore"]:
             if value := getattr(self, attr):
